// AP(r) Computer Science Marine Biology Simulation:
// The MouseDisplay class is based on the FishDisplay class
// copyright(c) 2002 College Entrance
// Examination Board (www.collegeboard.com).
//
// This class is free software; you can redistribute it and/or modify
// it under the terms of the GNU General Public License as published by
// the Free Software Foundation.
//
// This class is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU General Public License for more details.

import java.awt.Color;
import java.awt.Shape;
import java.awt.Component;
import java.awt.Graphics2D;
import java.awt.GradientPaint;
import java.awt.geom.Ellipse2D;
import java.awt.geom.GeneralPath;
import java.awt.geom.Arc2D;
import java.awt.geom.AffineTransform;

/**
 *  AP&reg; Computer Science Marine Biology Simulation:<br>
 *  A <code>MouseDisplay</code> draws a simple mouse using Java2D graphics.
 *  The fish will be drawn as an elliptical body and triangular
 *  head with two circles for eyes.
 *
 *  <p>
 *  The <code>MouseDisplay</code> class is based
 *  on the FishDisplay class which is
 *  copyright&copy; 2002 College Entrance Examination Board
 *  (www.collegeboard.com).
 *
 *  @author Barb Ericson ericson@cc.gatech.edu
 *  @author Julie Zelenski
 *  @author Alyce Brady
 *  @author Chris Nevison
 *  @version 1 Feb 2005
 **/

public class MouseDisplay extends AbstractDrawableDisplay
{    
    private static final double BODY_WIDTH = .3, BODY_LENGTH = .4;
    private static final double HEAD_WIDTH = .2, HEAD_LENGTH = .35;
    private static final double TAIL_WIDTH = .13, TAIL_LENGTH = .27;
    private static final double EYE_SIZE = .08;
    
    private static final int GRADIENT_SIZE = 50;
    private static final AffineTransform ATX = AffineTransform.getScaleInstance(GRADIENT_SIZE, GRADIENT_SIZE);
    
    private Shape bodyAndHead, eye1, eye2, tail;
    
    /** Constructs an object that knows how to draw simple fish.
     **/
    public MouseDisplay()
    {    
        buildPaths(BODY_WIDTH, BODY_LENGTH,
                   HEAD_WIDTH, HEAD_LENGTH, 
                   TAIL_WIDTH, TAIL_LENGTH, 
                   EYE_SIZE);
    }
    
    /** Sets up the paths used for the mouse body, head, tail, and eyes.
     *  Different parameters will change the proportions, and thereby
     *  control the "look" of the fish.  The various parameters should be 
     *  specified assuming the fish will occupy a cell of size (1, 1).
     */
    protected void buildPaths(double bodyWidth, double bodyLength, 
                              double headWidth, double headLength, 
                            double tailWidth, double tailLength, 
                            double eyeSize)
    {
      float overlap = (float) (headLength * 0.33);
      
        // Build a set of paths for a mouse facing North in a unit-length cell.
        // We will rotate/scale as needed later.
        
        float halfLength = (float)((bodyLength + 
                                   2 * tailLength) / 2);
        
        // The body is an ellipse of the given body width and length.
        // The ellipse is horizontally centered and slightly above vertical
        // center (to leave room for tail).
        Shape body = new Ellipse2D.Double(-bodyWidth/2, 
                                          -bodyLength/2,
                                          bodyWidth, bodyLength);
   
        // The mouse head is a triangle overlapping the north end of body.
        GeneralPath head = new GeneralPath();
        head.moveTo(-(float)(headWidth/2), (float) (-halfLength + headLength)); // lower left
        head.lineTo(0, -halfLength); // top of head
        head.lineTo((float)(headWidth/2), (float) (-halfLength + headLength)); // lower right
        head.closePath();
        
        // Join body and head together in one path.
        head.append(body, false);
        bodyAndHead = head;
        
        // the tail is an arc
        tail = new Arc2D.Double(0,bodyLength/2-tailLength,tailWidth*2,tailLength*2,180,210,Arc2D.OPEN);
    
        // The eyes are circles.
        eye1 = new Ellipse2D.Double(-headWidth/5 - eyeSize/2, -halfLength + headLength/3, eyeSize, eyeSize);
        eye2 = new Ellipse2D.Double(+headWidth/5 - eyeSize/2, -halfLength + headLength/3, eyeSize, eyeSize);
    }

    /** Draws the given Mouse.
     *  Fills a simple mouse with gradient paint using the paths created in
     *  the construtor. The Mouse is drawn facing North in a cell of
     *  size (1,1) centered around (0,0) on the drawing surface.
     *  (All scaling/rotating has been done beforehand).
     *  @param   mouse       mouse we want to draw
     *  @param   comp       component on which to draw
     *  @param   g2         drawing surface
     **/
    public void draw(Mouse mouse, Component comp, Graphics2D g2)
    {
        Color color = mouse.color();
        
       // Stroke outline of mouse body and head in slightly darker color.
        g2.setPaint(color.darker());  
        g2.draw(bodyAndHead);

        // Fill mouse body and head with gradient (scale up temporarily to get smooth dither).
        g2.scale(1.0/GRADIENT_SIZE, 1.0/GRADIENT_SIZE);  
        g2.setPaint(new GradientPaint(-GRADIENT_SIZE/4, -GRADIENT_SIZE/2, Color.white, 
                                       GRADIENT_SIZE/4, GRADIENT_SIZE/4, color));
        g2.fill(ATX.createTransformedShape(bodyAndHead));
        g2.scale(GRADIENT_SIZE, GRADIENT_SIZE);

        // Fill black circles for the eyes.
        g2.setPaint(Color.black);
        g2.fill(eye1);
        g2.fill(eye2);
        g2.draw(tail);
    }
    
    public void draw(Drawable drawable, Component comp, Graphics2D g2)
    {  
      draw((Mouse) drawable, comp, g2);
    }

}
