// AP(r) Computer Science Marine Biology Simulation:
// The EnvironmentController class is copyright(c) 2002 College Entrance
// Examination Board (www.collegeboard.com).
//
// This class is free software; you can redistribute it and/or modify
// it under the terms of the GNU General Public License as published by
// the Free Software Foundation.
//
// This class is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU General Public License for more details.

import java.awt.Color;
import java.awt.Dimension;
import java.awt.Toolkit;
import java.awt.event.ActionEvent;
import java.awt.event.MouseEvent;
import java.awt.event.MouseAdapter;
import javax.swing.JOptionPane;
import javax.swing.JFileChooser;

/**
 *  AP&reg; Computer Science Marine Biology Simulation:<br>
 *  An <code>EnvironmentController</code> sets up the controls for manipulating
 *  the Environment and responds to actions dispatched by
 *  those controls.
 *
 *  <p>
 *  The <code>EnvironmentController</code> class is
 *  copyright&copy; 2002 College Entrance Examination Board
 *  (www.collegeboard.com).
 *
 *  @author Julie Zelenski
 *  @author Barb Ericson ericson@cc.gatech.edu 
 *  @modified 
 *  @version 1 August 2002
 **/

public class EnvironmentController
{
    // constants for the three different seeding options
    public static final int DONT_RESET_SEED = 0, FIXED_SEED = 1, PROMPT_SEED = 2;

    // this is the direction each new drawable object will face when created
    private static final Direction START_FACING = Direction.EAST;
    
    private Environment env;
    private int seed, seedOption;

    private MBSGUIFrame parentFrame;
    private MBSDisplay display;
    private SimulationController simControl;
    private EnvFileChooser fileChooser;
    private CreateEnvDialog createEnvDialog;
    private DrawableToolbar toolbar;
        
    /** Creates a new EnvironmentController tied to the specified frame,
     *  display, and simulation controller.
     **/
    public EnvironmentController(MBSGUIFrame f, MBSDisplay disp, SimulationController sc)
    {
        display = disp;
        parentFrame = f;
        simControl = sc;
        toolbar = new DrawableToolbar();
        setDrawableToolbarVisible(false);
        display.addMouseListener(new MouseAdapter() {
            public void mousePressed(MouseEvent evt) { createDrawableForEvent(evt); }});
    }
    
    /** Returns the toolbar of drawable creation controls.
     **/
    public DrawableToolbar controlPanel()
    {
        return toolbar;
    }

    /** Private helper to change the Environment being operated on.  Messages other
     *  interested parties to update as well.
     **/
    private void setEnv(Environment newEnv)
    {
        env = newEnv;
        display.setEnv(env);
        simControl.setEnv(env, this);
        parentFrame.setEnvMenuItemsEnabled(true);
    }
    
    
    /** Creates a new empty environment.  Get environment type and
     *  dimensions via the CreateEnvDialog, and resets seed if needed.
     *  The drawable toolbar is automatically made visible to allow immediate
     *  editing of the new environment.
     **/
    public void createNewEnv()
    {   
        if (createEnvDialog == null)
            createEnvDialog = new CreateEnvDialog(parentFrame);
        Environment newEnv = createEnvDialog.showDialog();
        if (newEnv != null)
        {
            resetSeedIfNeeded();
            setDrawableToolbarVisible(true);
            setEnv(newEnv);
        }
    }
    
    /** Enables editing of an existing environment. Stops the 
     *  simulation if running and then makes the drawable toolbar visible.
     **/
    public void editEnv()
    {        
        simControl.stop();
        setDrawableToolbarVisible(true);
    }

    /** Reads a new environment from a saved data file. Gets file via
     *  file chooser dialog, reads contents using MBSDataFileHandler.
     *  On error, brings up dialog and continues using previous environment.
     **/
    public void openFile()
    {
        if (fileChooser == null) fileChooser = new EnvFileChooser();
        if (fileChooser.showOpenDialog(parentFrame) != JFileChooser.APPROVE_OPTION)
            return;
            
        Environment newEnv;
        try 
        {
            MBSDataFileHandler handler = new MBSDataFileHandler(fileChooser.getSelectedFile());
            resetSeedIfNeeded();
            newEnv = handler.readMBSEnv(fileChooser.boundedClass(), fileChooser.unboundedClass());
        }
        catch (Exception ex)
        {
            JOptionPane.showMessageDialog(parentFrame, "Unable to read environment from file.\n"
                        + "Reason: " + ex, "Error reading file", JOptionPane.ERROR_MESSAGE);
            return;
        }
        setDrawableToolbarVisible(false);
        setEnv(newEnv);
    }
    
    
     /** Saves an environment to a data file. Gets file via
     *  file chooser dialog, writes contents using MBSDataFileHandler.
     *  On error, brings up dialog and discards request.
     **/
    public void saveFile()
    {
        if (fileChooser == null) fileChooser = new EnvFileChooser();
        simControl.stop();
        if (fileChooser.showSaveDialog(parentFrame) != JFileChooser.APPROVE_OPTION)
            return;
            
        try 
        {
            MBSDataFileHandler handler = new MBSDataFileHandler(fileChooser.getSelectedFile());
            handler.writeMBSEnv(env);
        }
        catch (Exception ex)
        {
            JOptionPane.showMessageDialog(parentFrame, "Unable to save environment to file.\n"
                        + "Reason: " + ex, "Error saving file", JOptionPane.ERROR_MESSAGE);
        }
    }
            
    /** Changes the current run option to one of DONT_RESET, FIXED_SEED, or
     *  PROMPT_SEED to specify the seed behavior.   The seed option now
     *  in effect is returned. Thist usually will be the same as newOption
     *  but in cases where the change failed to go through (user cancelled
     *  dialog), the retun value will be the existing unchanged option.
     *  @param newOption the seed option to use for new/open environments
     *  @returns the seed option settled on
     **/
    public int setSeedOption(int newOption)
    {
        if (newOption == FIXED_SEED && !promptUserForSeed())
            return seedOption; // leave option unchanged
        seedOption = newOption;
        if (seedOption == PROMPT_SEED)
            JOptionPane.showMessageDialog(parentFrame, 
            "You will be prompted for a new seed when opening or creating an environment.", 
            "Prompt for seed", JOptionPane.INFORMATION_MESSAGE);
        return seedOption;
    }
    
    /** If seed is being reset for each new/open environment, this
     *  method handles re-setting the generator appropriately.
     **/
    private void resetSeedIfNeeded()
    {
        switch (seedOption) 
        {
            case PROMPT_SEED:
                promptUserForSeed(); // fall-through to next case
            case FIXED_SEED:
                RandNumGenerator.getInstance().setSeed(seed); 
        }
    }


    /** Runs a dialog asking the user to input a new seed. If
     * successful, updates seed and returns true, else returns false.
     **/
    private boolean promptUserForSeed() 
    {
        // dialog suggests 17 because that's the value mentioned in the narrative
        int suggestion = seed == 0 ? 17 : seed;
        String seedStr = (String)JOptionPane.showInputDialog(parentFrame, 
                        "Enter seed for random number generator:", "Input",
                        JOptionPane.QUESTION_MESSAGE, null, null, "" + suggestion);
        if (seedStr != null)
            try
            {
                seed = Integer.parseInt(seedStr.trim());
                return true;
            } 
            catch (Exception ex) { Toolkit.getDefaultToolkit().beep(); }
        return false;
    }

    /** Shows/hides the drawable toolbar. In order to avoid disrupting the
     *  current layout, resize the parent frame by the amount need to
     *  grow/shrink to accommodate the change in window size without
     *  resizing any other components.
     */
    public void setDrawableToolbarVisible(boolean visible)
    {
        if (toolbar.isVisible() != visible) 
        {
            toolbar.setVisible(visible);
            java.awt.Dimension cur = parentFrame.getSize();
            Dimension change;
            if (toolbar.getOrientation() == toolbar.HORIZONTAL)
                change = new Dimension(0, (visible ? 1 : -1) * toolbar.getPreferredSize().height);
            else
                change = new Dimension((visible ? 1 : -1) * toolbar.getPreferredSize().width, 0);
            parentFrame.setSize(new Dimension(cur.width + change.width, cur.height + change.height));
            parentFrame.validate(); // force immediate validation
        }
    }
    
    /** Callback on mousePressed when editing an environment. Given location
     *  under mouse, either rotate, add, or remove drawable from that location.
     */
    private void createDrawableForEvent(MouseEvent evt)
    {
        Location loc = display.locationForPoint(evt.getPoint());
        if (loc != null && toolbar.isVisible())
        {
            Drawable f = (Drawable)env.objectAt(loc);
            Class selectedClass = toolbar.currentDrawableClass();
            if (f == null || !f.getClass().equals(selectedClass))
            {
                if (f != null) env.remove(f); // remove previous drawable
                constructDrawable(selectedClass, env, loc, START_FACING, toolbar.currentColor());
            }
            else
            {
                int turningAngle = Direction.FULL_CIRCLE/env.numAdjacentNeighbors();
                Direction newFacing = f.direction().toRight(turningAngle);
                if (newFacing.equals(START_FACING)) // this turn completes circle
                    env.remove(f); 
                else
                    f.changeDirection(newFacing);
            }
            display.repaint();
        }
    }
    
    
    /** Private helper for drawable creation with error-reporting.
     */
    private void constructDrawable(Class cls, Environment env, Location loc, 
                                Direction dir, Color color)
    {
        try
        {
            if (MBSFactory.hasFourArgCtor(cls))
                MBSFactory.constructEnvObject(cls, env, loc, dir, color);
            else
                MBSFactory.constructEnvObject(cls, env, loc, dir);
        }
        catch (Exception e)
        {                       // If we get here, it's an error.
            JOptionPane.showMessageDialog(parentFrame, "Cannot construct " + cls.getName() +
                                " with Environment, Location, Direction, and Color parameters.\n"
                                + "Reason: " + e,
                                "Error constructing drawable", JOptionPane.ERROR_MESSAGE);
        }
    }

    
}
