// AP(r) Computer Science Marine Biology Simulation:
// The SimulationController class is copyright(c) 2002 College Entrance
// Examination Board (www.collegeboard.com).
//
// This class is free software; you can redistribute it and/or modify
// it under the terms of the GNU General Public License as published by
// the Free Software Foundation.
//
// This class is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU General Public License for more details.

import java.awt.Dimension;
import java.awt.Component;
import java.awt.Toolkit;
import java.awt.event.ActionListener;
import java.awt.event.ActionEvent;
import javax.swing.event.ChangeEvent;
import javax.swing.event.ChangeListener;
import javax.swing.*;


/**
 *  AP&reg; Computer Science Marine Biology Simulation:<br>
 *  A <code>SimulationController</code> sets up the controls for manipulating
 *  the <code>Simulation</code> and responds to actions dispatched by
 *  those controls.
 *
 *  <p>
 *  The <code>SimulationController</code> class is
 *  copyright&copy; 2002 College Entrance Examination Board
 *  (www.collegeboard.com).
 *
 *  @author Julie Zelenski
 *  @author Barb Ericson Jan 2005 
 *  @version 1 August 2002
 **/

public class SimulationController  
{
    public static final int INDEFINITE = 0, FIXED_STEPS = 1, PROMPT_STEPS = 2;

    private static final int MIN_DELAY_MSECS = 10, MAX_DELAY_MSECS = 1000;
    private static final int INITIAL_DELAY = MIN_DELAY_MSECS +
                                    (MAX_DELAY_MSECS-MIN_DELAY_MSECS)/2;
    
    private Simulation simulation;
    private Timer timer;
    private JButton stepButton, runButton, stopButton;
    private JComponent controlPanel;
    private MBSDisplay display;
    private EnvironmentController envControl;
    private MBSGUIFrame parentFrame;
    private int runOption, numStepsToRun, numStepsSoFar;


    /** Creates a new SimulationController tied to the specified display
     *  and gui frame.
     *  @param  parent   the MBSGUIFrame for the simulation window
     *  @param  disp   the EnvDisplay to use for the simulation
    **/
    public SimulationController(MBSGUIFrame parent, MBSDisplay disp)
    {
        display = disp;
        parentFrame = parent;
        makeControls(); 
        timer = new Timer(INITIAL_DELAY, new ActionListener() {
            public void actionPerformed(ActionEvent evt) { step(); }});
   }
    
    
    
    /** Advances the simulation one step. 
     **/
    public void step()
    { 
        envControl.setDrawableToolbarVisible(false);
        simulation.step(); 
        if (++numStepsSoFar == numStepsToRun)
            stop();
    }
    
    /** Starts a timer to repeatedly step the simulation at
     *  the speed currently indicated by the speed slider up
     *  Depending on the run option, it will either step the
     *  the simulation some fixed number of steps or indefinitely
     *  until stopped.
     **/
    public void run() 
    { 
        if (runOption == PROMPT_STEPS)
            if (!promptUserForStepCount()) return;
        display.setToolTipsEnabled(false); // hide tool tips while running
        parentFrame.setRunMenuItemsEnabled(false);
        stopButton.setEnabled(true);
        stepButton.setEnabled(false);
        runButton.setEnabled(false);
        numStepsSoFar = 0;
        timer.start(); 
   }
  
    /** Stops any existing timer currently stepping the simulation.
     **/
    public void stop()
    {
        display.setToolTipsEnabled(true);
        parentFrame.setRunMenuItemsEnabled(true);
        timer.stop(); 
        stopButton.setEnabled(false);
        runButton.setEnabled(true); 
        stepButton.setEnabled(true); 
    }
    
    /** Changes the current run option to one of INDEFINITE, FIXED_STEPS, or
     *  PROMPT_STEPS to specify the run behavior. The text of the Run
     *  button is updated to show the current option.  The run option
     *  in effect is returned. This usually will be the same as newOption
     *  but in cases where the change failed to go through (user cancelled
     *  dialog), the retun value will be the existing unchanged option.
     *  @param newOption the run option to use for the simuiation
     *  @returns the run option settled on by the simulation.
     **/
     public int setRunOption(int newOption)
    {
        switch (newOption)
        {
            case INDEFINITE:
                runButton.setText("Run");
                numStepsToRun = INDEFINITE;
                break;
            case FIXED_STEPS:
                if (!promptUserForStepCount()) 
                    return runOption; // leave option unchanged
                runButton.setText("Run for " + numStepsToRun + " step" + ((numStepsToRun == 1)? "" :"s"));
                break;
            case PROMPT_STEPS:
                runButton.setText("Run...");
        }
        runOption = newOption;
        return runOption;
    }
    
    
    /** Runs a dialog asking the user to input a number of steps
     *  for the run behavior. If successful, updates numStepsToRun
     *  and returns true, else returns false.
     **/
    private boolean promptUserForStepCount() 
    {
        int suggested = numStepsToRun > 0 ? numStepsToRun : 10;
        String response = (String)JOptionPane.showInputDialog(parentFrame, 
                        "Enter number of steps:", "Input",
                        JOptionPane.QUESTION_MESSAGE, null, null, "" + suggested);
        if (response != null)
            try
            {
                int userEntered = Integer.parseInt(response.trim());
                if (userEntered <= 0)
                    throw new NumberFormatException();
                numStepsToRun = userEntered;
                return true;
            } 
            catch (Exception ex) { Toolkit.getDefaultToolkit().beep(); }
        return false;
    }


    /** Builds the panel with the various simulation controls (buttons and 
     *  slider).
     **/
    private void makeControls()
    {
        controlPanel = new JPanel();
        controlPanel.setLayout(new BoxLayout(controlPanel, BoxLayout.X_AXIS));
        controlPanel.setBorder(BorderFactory.createTitledBorder("Simulation"));
        controlPanel.add(Box.createRigidArea(new Dimension(5, 0)));
        
        controlPanel.add(stepButton = new JButton("Step"));
        controlPanel.add(runButton = new JButton("Run"));
        controlPanel.add(stopButton = new JButton("Stop"));
        runButton.setEnabled(false);
        stepButton.setEnabled(false);
        stopButton.setEnabled(false);
        
        controlPanel.add(Box.createRigidArea(new Dimension(5, 0)));
        controlPanel.add(new JLabel("Slow"));
        JSlider speedSlider = new JSlider(MIN_DELAY_MSECS, MAX_DELAY_MSECS, INITIAL_DELAY);
        speedSlider.setInverted(true);
        speedSlider.setPreferredSize(new Dimension(100, speedSlider.getPreferredSize().height));
        speedSlider.setMaximumSize(speedSlider.getPreferredSize());
        controlPanel.add(speedSlider);
        controlPanel.add(new JLabel("Fast"));
        controlPanel.add(Box.createRigidArea(new Dimension(5, 0)));

        stepButton.addActionListener(new ActionListener() {
            public void actionPerformed(ActionEvent e) { step(); }});
        runButton.addActionListener(new ActionListener() {
            public void actionPerformed(ActionEvent e) { run(); }});
        stopButton.addActionListener(new ActionListener() {
            public void actionPerformed(ActionEvent e) { stop(); }});
        speedSlider.addChangeListener(new ChangeListener() {
            public void stateChanged(ChangeEvent evt) {
                timer.setDelay(((JSlider)evt.getSource()).getValue());
            }});
    }
    

    /** Returns the panel containing the simulation controls.
     *  @return the control panel
     **/
    public JComponent controlPanel()
    {
        return controlPanel;
    }

    /** Sets the current environment and environment controller.
     *  Whenever the environment changes (upon creating a new one, 
     *  reading environment from a file, etc.) this method must be called
     *  to reconfigure the simulation to operate on this new environment.
     *  @param newEnv the new environment  
     *  @param ec the environment controller
     **/
    public void setEnv(Environment newEnv, EnvironmentController ec)
    {
        stop();
        simulation = new Simulation(newEnv, display);
        envControl = ec;
    }


}
